import * as plugins from '../plugins.js';
import * as data from '../data/index.js';
import * as userInterfaces from '../data/user.js';

export interface IReq_GetEnvBundle extends plugins.typedrequestInterfaces.implementsTR<
  plugins.typedrequestInterfaces.ITypedRequest,
  IReq_GetEnvBundle
> {
  method: 'getEnvBundle';
  request: {
    authorization: string;
    /**
     * specify this if you want to get a warning, if the envBundle is for an unexpected environment
     */
    environment?: string;
  };
  response: {
    envBundle: data.IEnvBundle;
  };
}

export interface IReq_Admin_LoginWithUsernameAndPassword extends plugins.typedrequestInterfaces.implementsTR<
  plugins.typedrequestInterfaces.ITypedRequest,
  IReq_Admin_LoginWithUsernameAndPassword
> {
  method: 'adminLoginWithUsernameAndPassword';
  request: {
    username: string;
    password: string;
  };
  response: {
    identity: userInterfaces.IIdentity;
  }
}

export interface IReq_Admin_GetConfigBundlesAndSecretGroups extends plugins.typedrequestInterfaces.implementsTR<
  plugins.typedrequestInterfaces.ITypedRequest,
  IReq_Admin_GetConfigBundlesAndSecretGroups
> {
  method: 'adminGetConfigBundlesAndSecretGroups';
  request: {
    identity: userInterfaces.IIdentity;
  };
  response: {
    secretBundles: data.ISecretBundle[];
    secretGroups: data.ISecretGroup[];
  };

}

export interface IReq_Admin_CreateConfigBundlesAndSecretGroups extends plugins.typedrequestInterfaces.implementsTR<
  plugins.typedrequestInterfaces.ITypedRequest,
  IReq_Admin_CreateConfigBundlesAndSecretGroups
> {
  method: 'adminCreateConfigBundlesAndSecretGroups';
  request: {
    identity: userInterfaces.IIdentity;
    secretBundles: data.ISecretBundle[];
    secretGroups: data.ISecretGroup[];
  };
  response: {
    ok: boolean;
  };
}

export interface IReq_Admin_UpdateConfigBundlesAndSecretGroups extends plugins.typedrequestInterfaces.implementsTR<
  plugins.typedrequestInterfaces.ITypedRequest,
  IReq_Admin_UpdateConfigBundlesAndSecretGroups
> {
  method: 'adminUpdateConfigBundlesAndSecretGroups';
  request: {
    identity: userInterfaces.IIdentity;
    configBundles: data.ISecretBundle[];
    secretGroups: data.ISecretGroup[];
  };
  response: {
    ok: boolean;
  };
}

export interface IReq_Admin_DeleteConfigBundlesAndSecretGroups extends plugins.typedrequestInterfaces.implementsTR<
  plugins.typedrequestInterfaces.ITypedRequest,
  IReq_Admin_DeleteConfigBundlesAndSecretGroups
> {
  method: 'adminDeleteConfigBundlesAndSecretGroups';
  request: {
    identity: userInterfaces.IIdentity;
    secretBundleIds: string[];
    secretGroupIds: string[];
  };
  response: {
    ok: boolean;
  };
}
